/******************************************************************************/
/*  INTR.C - TMS320C6x Peripheral Support Library Interrupt Support           */
/*                                                                            */
/*     This module provides routines to control and initialize                */
/*     interrupt processing facilities on the C6x.                            */
/*                                                                            */
/*  FUNCTIONS:                                                                */
/*     intr_reset()         - Reset interrupt registers to default values     */
/*     intr_init()          - Interrupt initialization                        */
/*     intr_isn()           - Assign ISN to CPU interrupt                     */
/*     intr_get_cpu_intr()  - Return CPU interrupt assigned to ISN            */
/*     intr_map()           - Place ISN in interrupt multiplexor register     */
/*                                                                            */
/*  STATIC FUNCTIONS:                                                         */
/*    None.                                                                   */
/*                                                                            */
/*  GLOBAL VARIABLES DEFINED:                                                 */
/*    isr_jump_table        - Interrupt jump table                            */
/*                                                                            */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* INCLUDES                                                                   */
/*----------------------------------------------------------------------------*/
#include <stdio.h>

#include "intr_c6713.h"  /* change back to "intr.h" when all is working with RTDX */

/*----------------------------------------------------------------------------*/
/* GLOBAL VARIABLES                                                           */
/*----------------------------------------------------------------------------*/
/* Interrupt Service Routine Jump Table                                       */
unsigned int isr_jump_table[16] = {
                        (unsigned int) c_int00,
                        (unsigned int) (&NMI),
                        (unsigned int) (&RESV1),
                        (unsigned int) (&RESV2), /* Used by RTDX */
                        (unsigned int) (&unexp_int04),
                        (unsigned int) (&unexp_int05),
                        (unsigned int) (&unexp_int06),
                        (unsigned int) (&unexp_int07),
                        (unsigned int) (&unexp_int08),
                        (unsigned int) (&unexp_int09),
                        (unsigned int) (&unexp_int10),
                        (unsigned int) (&unexp_int11), /* Reserved for RTDX */
                        (unsigned int) (&unexp_int12), /* Reserved for RTDX */
                        (unsigned int) (&unexp_int13),
                        (unsigned int) (&unexp_int14),
                        (unsigned int) (&unexp_int15 )};

/*----------------------------------------------------------------------------*/
/* FILE LOCAL (STATIC) VARIABLES                                              */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* FILE LOCAL (STATIC) PROTOTYPES                                             */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* FUNCTIONS                                                                  */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* INTR_RESET - reset interrupt registers to default values                   */
/*----------------------------------------------------------------------------*/ 
void intr_reset(void)
{

  /* disable global interrupts                                                */
  LOAD_REG_FIELD(CSR,0,0,2);

  /* disable interrupts                                                       */
  SET_REG(IER,1);
  SET_REG(ICR,0xFFF0);

  /* default external interrupts to rising-edge triggered                     */
  REG_WRITE(EXTERNAL_INTR_POL_ADDR,0);

  /* reset interrupt multiplexers                                             */
  INTR_MAP_RESET();

  /* initialize interrupt service table pointer                               */
  SET_REG(ISTP, (unsigned int) istb);
}

/*----------------------------------------------------------------------------*/
/* INTR_INIT - initialize Interrupt Service Table Pointer                     */
/*                                                                            */
/*   This function initializes the ISTP based upon the global vec_table which */
/*   is resolved at link time. Refer to linker command file for this value    */
/*----------------------------------------------------------------------------*/ 
void intr_init(void)
{
  
    SET_REG(ISTP,(unsigned int) istb);
}

/*----------------------------------------------------------------------------*/
/* INTR_HOOK - hooks an ISR to interrupt.                                     */
/*----------------------------------------------------------------------------*/
void intr_hook(void (*fp)(void),int cpu_intr)
{
  if ((cpu_intr > 0) && (cpu_intr < 16))
    *((unsigned int *)isr_jump_table + cpu_intr)= (unsigned int)fp;
}


/*----------------------------------------------------------------------------*/
/* INTR_MAP() - Map interrupt source (isn) to cpu interrupt (cpu_intr).       */
/*                                                                            */
/*   This function loads the isn value into the INTSEL field of the           */
/*   appropriate Interrupt Multiplexer register indicated by cpu_intr         */
/*----------------------------------------------------------------------------*/
void intr_map(int cpu_intr,int isn)
{

  int intsel;
  int intr_field;
  int sel;

  sel= 0;
  intr_field= cpu_intr;

  if (cpu_intr > CPU_INT9)
  {
    sel=1;
	  intr_field -= 6;
  }

  intsel = (intr_field - 4) * 5;

  if (intsel > 10)
    intsel++;

  INTR_SET_MAP(intsel,isn,sel);
}

/*----------------------------------------------------------------------------*/
/* INTR_ISN() - return isn currently mapped to cpu interrupt              */
/*----------------------------------------------------------------------------*/
int intr_isn(int cpu_intr)
{
  int intsel;
  int intr_field;
  int sel;
  int isn;

  sel= 0;
  intr_field= cpu_intr;

  if (cpu_intr > CPU_INT9)
  {
     sel=1;
     intr_field -= 6;
  }

  intsel = (intr_field - 4) * 5;

  if (intsel > 10)
     intsel++;

  isn= INTR_GET_ISN(intsel,sel);
  return(isn);

}

/*----------------------------------------------------------------------------*/
/* intr_get_cpu_intr() - return cpu interrupt corresponding to isn in         */
/*                       interrupt selecter register.  If the isn is not      */
/*                       mapped, return -1                                    */
/*----------------------------------------------------------------------------*/
int intr_get_cpu_intr(int isn)
{
  int i;

  for (i= CPU_INT4;i<=CPU_INT15;i++)
  {
    if (intr_isn(i) == isn)
      return(i);
  }
  return(-1);
}

